 /**************************************************************************                      
 *							 
 *       Copyright (C) 2003 Freescale Semiconductor, Inc. 
 *           and 2000-2002 Viola Systems Ltd. 
 *       All Rights Reserved
 *				 
 * File Name      : Main.c
 * Description    : UDP demo
 * 
 * *** NOTE: This project is for demonstration purposes only. 
 *                 
 * Version : 1.0
 *
 ***************************************************************************/

#include "debug.h"
#include "datatypes.h"
#include "timers.h"
#include "system.h"
#include "ethernet.h"
#include "arp.h"
#include "ip.h"
#include "tcp_ip.h"
#include "http_server.h"
#include "smtp_client.h"

#include "ne64driver.h"
#include "ne64api.h"
#include "mBuf.h"
#include "ne64config.h"
#include "address.h"
#include "MC9S12NE64.h"
/* Including used modules for compiling procedure */

/* Network Interface definition. Must be somewhere so why not here? :-)*/
struct netif localmachine;

extern void RTI_Enable (void);
extern void porth_isr_handler (void);

extern	tU08	gotlink; 

#if USE_SWLED
tU16 LEDcounter=0;
#endif


//============================================================
tU08 OldSwitchValue=255;
tU16 Pot=0;
tU16 OldPot=1050;
tU08 OldB1=255; 
tU08 OldB2=255;
tU08 OldB3=255;
tU08 OldB4=255;

//============================================================
#pragma CODE_SEG NON_BANKED
interrupt void PortHInterrupt (void)
{			   
  porth_isr_handler();
}
#pragma CODE_SEG DEFAULT


//============================================================
//Initialize ATD
//============================================================
void ATD_init(void)
{
    ATDCTL2 =  ATDCTL2_ADPU_MASK | ATDCTL2_AFFC_MASK;
    ATDCTL3_S1C = 8;    // 8 ch seq. 
    ATDCTL3_FIFO = 0;    // no FIFO
    ATDCTL3_FRZ = 3;     // Freeze immediately in BDM  
    ATDCTL4 =  ATDCTL4_PRS2_MASK |ATDCTL4_PRS1_MASK | ATDCTL4_PRS0_MASK;
    ATDCTL4 = ATDCTL4 & ~ATDCTL4_SRES8_MASK; //10 bit
    ATDCTL5 = ATDCTL5_SCAN_MASK;
}


//============================================================
// Initialize Port for LEDs, Switch, and Buttons
//============================================================
void demoinit(void) 
 {
     //LEDS
    DDRG_DDRG0 = 1;					//1:output
    DDRG_DDRG1 = 1;					//1:output
    DDRG_DDRG2 = 1;					//1:output
    DDRG_DDRG3 = 1;					//1:output
 		PTG_PTG0 = 1;	//turn off		
 		PTG_PTG1 = 1;	//turn off		
 		PTG_PTG2 = 1;	//turn off		
 		PTG_PTG3 = 1;	//turn off		

    //SWITCH (RUN/LOAD)		 0:input
    DDRG_DDRG4 = 0;

    //BUTTON2 
    DDRH_DDRH4 = 0;
    PIEH_PIEH4 = 1;      //PIEH4 Interrupt Enable

    DDRH_DDRH5 = 0;			 //0:input
    DDRH_DDRH6 = 0;			 //0:input

    
 }


//============================================================
/* main */
//============================================================
void main(void)
{
  INT16 len;

  /* System clock initialization */
  CLKSEL=0;
  CLKSEL_PLLSEL = 0;                   /* Select clock source from XTAL */
  PLLCTL_PLLON = 0;                    /* Disable the PLL */
  SYNR = 0;                           /* Set the multiplier register */
  REFDV = 0;                          /* Set the divider register */
  PLLCTL = 192;
  PLLCTL_PLLON = 1;                    /* Enable the PLL */
  while(!CRGFLG_LOCK);                 /* Wait */
  CLKSEL_PLLSEL = 1;                   /* Select clock source from PLL */

  INTCR_IRQEN = 0;                     /* Disable the IRQ interrupt. IRQ interrupt is enabled after CPU reset by default. */
	
    init();
    demoinit();
    ATD_init();
   	/* IP address */
 	  localmachine.localip = *((UINT32 *)ip_address);
   	/* Default gateway */
   	localmachine.defgw   = *((UINT32 *)ip_gateway);
   	/* Subnet mask */
   	localmachine.netmask = *((UINT32 *)ip_netmask);

   	/* Ethernet (MAC) address */
   	localmachine.localHW[0] = hard_addr[0];
   	localmachine.localHW[1] = hard_addr[1];
   	localmachine.localHW[2] = hard_addr[2];
   	localmachine.localHW[3] = hard_addr[3];
   	localmachine.localHW[4] = hard_addr[4];
   	localmachine.localHW[5] = hard_addr[5];

	/* Init system services		*/    
	timer_pool_init();
		
    /* Initialize all buffer descriptors */
	mBufInit ();

	/*interrupts can be enabled AFTER timer pool has been initialized */
    	
  /* Initialize all network layers	*/
  EtherInit();
  
  //Initialize required network protocols 	
  arp_init();
  (void)udp_init();

	udp_demo_init();
  (void)tcp_init();
	(void)https_init ();
	
	//Enable RTI
	RTI_Enable ();

	/* main loop */
	DEBUGOUT(">>>>>>>>>Entering to MAIN LOOP>>>>>>>>>\n\r");
	for (;;)
	{
#if USE_SWLED
      UseSWLedRun();
#endif
	  if (gotlink) {
	  
	  	/* take care of watchdog stuff */
   		/* Try to receive Ethernet Frame	*/    	
   		if( NETWORK_CHECK_IF_RECEIVED() == TRUE ) {  		   		
   			switch( received_frame.protocol)
			  {
   				case PROTOCOL_ARP:
					  process_arp (&received_frame);
	    			break;
   				case PROTOCOL_IP:
   					len = process_ip_in(&received_frame);
  					if(len < 0)
     						break;
     					switch (received_ip_packet.protocol)
  					  {
     						case IP_ICMP:
      						process_icmp_in (&received_ip_packet, len);
  							  break;
     						case IP_UDP:
                  process_udp_in (&received_ip_packet,len);   							
                  break;
    						case IP_TCP:
      						process_tcp_in (&received_ip_packet, len);				
     							break;
     						default:
  	  						break;
    					}
       				break;
				  default:
   				  break;
			  }
			/* discard received frame */    		
   			NETWORK_RECEIVE_END();
   		}
    arp_manage();
 
		/* Application main loops */
		/* TCP/IP stack Periodic tasks here... */
    udp_demo_run();
	  tcp_poll();
    https_run ();
    
	  } 
	  else {
     		PTG_PTG0 = 1;	//turn off LED1		
     		PTG_PTG1 = 1;	//turn off LED2		  
	  }
	}
}